/*
 * U-Boot and JzRISC LCD controller definitions
 *
 * Copyright (C) 2001 Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 * Copyright (C) 2005-2007, Ingenic Semiconductor Inc.
 * Copyright (C) Xiangfu Liu <xiangfu@sharism.cc>
 * Copyright (C) 2015, 2016, 2017 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#ifndef __JZLCD_H__
#define __JZLCD_H__

#include <stdint.h>

/* Framebuffer characteristics. */

struct jzfb_info {
	uint32_t cfg;	/* panel mode and pin usage etc. */
	uint16_t w;	/* display width in pixels */
	uint16_t h;	/* display height in pixels */
	uint8_t bpp;	/* bits per pixel */
	uint32_t fclk;	/* frame clock */
	uint32_t hsw;	/* hsync width, in pixel clock */
	uint32_t vsw;	/* vsync width, in line count */
	uint32_t elw;	/* end of line, in pixel clock */
	uint32_t blw;	/* begin of line, in pixel clock */
	uint32_t efw;	/* end of frame, in line count */
	uint32_t bfw;	/* begin of frame, in line count */
};

/* LCD controller stucture for jz4740. */

struct jz_fb_dma_descriptor {
	struct jz_fb_dma_descriptor *fdadr;	/* frame descriptor address register */
	uint32_t fsadr;				/* frame source address register */
	uint32_t fidr;				/* frame identifier register */
	uint32_t ldcmd;				/* command register */
};

/* Framebuffer and controller memory information. */

struct jz_mem_info {

	/* DMA descriptor references (updated for transfers). */

	struct jz_fb_dma_descriptor *fdadr0;	/* physical address of frame descriptor */
	struct jz_fb_dma_descriptor *fdadr1;	/* physical address of frame/palette descriptor */

	/* DMA descriptor references (indicating allocated regions). */

	struct jz_fb_dma_descriptor *dmadesc_fb0;
	struct jz_fb_dma_descriptor *dmadesc_fb1;
	struct jz_fb_dma_descriptor *dmadesc_palette;

	uint32_t screen;			/* address of first frame buffer (base of memory used) */
	uint32_t palette;			/* address of palette memory */
	uint32_t total;				/* total memory used */
};

/* Display characteristics and memory resources. */

typedef struct vidinfo {
	struct jzfb_info *jz_fb;		/* framebuffer and panel properties */
	struct jz_mem_info jz_mem;		/* framebuffer memory information */
	void *lcd;				/* address of LCD controller registers */
} vidinfo_t;

/* Public functions. */

uint32_t jz4740_lcd_get_total_size(vidinfo_t *vid);
uint32_t jz4740_lcd_get_pixel_clock(vidinfo_t *vid);
void jz4740_lcd_ctrl_init(void *lcd_base, void *fb_vaddr, vidinfo_t *vid);
void jz4740_lcd_hw_init(vidinfo_t *vid);
void jz4740_lcd_dma_init(vidinfo_t *vid);
void lcd_set_bpp(uint8_t bpp);
uint32_t lcd_ctrl_init();
void lcd_enable();
void lcd_disable();

/* Alignment/rounding macros. */

#define ALIGN(x,a)              __ALIGN_MASK((x),(typeof(x))(a)-1)
#define __ALIGN_MASK(x,mask)    (((x)+(mask))&~(mask))

/* Transfer and display types. */

#define MODE_MASK		0x0f
#define MODE_TFT_GEN		0x00
#define MODE_TFT_SHARP		0x01
#define MODE_TFT_CASIO		0x02
#define MODE_TFT_SAMSUNG	0x03
#define MODE_CCIR656_NONINT	0x04
#define MODE_CCIR656_INT	0x05
#define MODE_STN_COLOR_SINGLE	0x08
#define MODE_STN_MONO_SINGLE	0x09
#define MODE_STN_COLOR_DUAL	0x0a
#define MODE_STN_MONO_DUAL	0x0b
#define MODE_8BIT_SERIAL_TFT    0x0c

#define MODE_TFT_18BIT          (1<<7)

#define STN_DAT_PIN1	(0x00 << 4)
#define STN_DAT_PIN2	(0x01 << 4)
#define STN_DAT_PIN4	(0x02 << 4)
#define STN_DAT_PIN8	(0x03 << 4)
#define STN_DAT_PINMASK	STN_DAT_PIN8

#define STFT_PSHI	(1 << 15)
#define STFT_CLSHI	(1 << 14)
#define STFT_SPLHI	(1 << 13)
#define STFT_REVHI	(1 << 12)

#define SYNC_MASTER	(0 << 16)
#define SYNC_SLAVE	(1 << 16)

#define DE_P		(0 << 9)
#define DE_N		(1 << 9)

#define PCLK_P		(0 << 10)
#define PCLK_N		(1 << 10)

#define HSYNC_P		(0 << 11)
#define HSYNC_N		(1 << 11)

#define VSYNC_P		(0 << 8)
#define VSYNC_N		(1 << 8)

#define DATA_NORMAL	(0 << 17)
#define DATA_INVERSE	(1 << 17)

/* LCD register base. */

#define LCD_BASE_KSEG1		0xB3050000

/* Register offsets. */

#define LCD_CFG			0x00 /* LCD Configure Register */
#define LCD_VSYNC		0x04 /* Vertical Synchronize Register */
#define LCD_HSYNC		0x08 /* Horizontal Synchronize Register */
#define LCD_VAT			0x0c /* Virtual Area Setting Register */
#define LCD_DAH			0x10 /* Display Area Horizontal Start/End Point */
#define LCD_DAV			0x14 /* Display Area Vertical Start/End Point */
#define LCD_PS			0x18 /* PS Signal Setting */
#define LCD_CLS			0x1c /* CLS Signal Setting */
#define LCD_SPL			0x20 /* SPL Signal Setting */
#define LCD_REV			0x24 /* REV Signal Setting */
#define LCD_CTRL		0x30 /* LCD Control Register */
#define LCD_STATE		0x34 /* LCD Status Register */
#define LCD_IID			0x38 /* Interrupt ID Register */
#define LCD_DA0			0x40 /* Descriptor Address Register 0 */
#define LCD_SA0			0x44 /* Source Address Register 0 */
#define LCD_FID0		0x48 /* Frame ID Register 0 */
#define LCD_CMD0		0x4c /* DMA Command Register 0 */
#define LCD_DA1			0x50 /* Descriptor Address Register 1 */
#define LCD_SA1			0x54 /* Source Address Register 1 */
#define LCD_FID1		0x58 /* Frame ID Register 1 */
#define LCD_CMD1		0x5c /* DMA Command Register 1 */

/* Palette buffer (LCD_CMDx.PAL). */

#define LCD_CMD_PAL		(1 << 28)

#endif /* __JZLCD_H__ */
